#ifndef HID_H_
#define HID_H_

#ifdef __USE_DRIVER
#include "lpc_types.h"
#endif

#include "usb.h"

#define HID_CLASS				0x03

// Descriptor Types
#define HID_DESCRIPTOR			0x21
#define REPORT_DESCRIPTOR		0x22
#define PHYSICAL_DESCRIPTOR		0x23

// HID Class Requests
#define GET_REPORT				0x01
#define GET_IDLE				0x02
#define GET_PROTOCOL			0x03
#define SET_REPORT				0x09
#define SET_IDLE				0x0A
#define SET_PROTOCOL			0x0B

#define REPORT_INPUT			0x01
#define REPORT_OUTPUT			0x02
#define REPORT_FEATURE			0x03

// 6.2.1 HID Descriptor Seite 22 in HID-Spec
struct HIDDescr {
	uint8_t		bLength;				//< Size of the Descriptor in Bytes
	uint8_t		bDescriptorType;		//< Device Descriptor (0x21)
	uint16_t	bcdHID;					//< Numeric expression identifying the HID Class Specification release.
	uint8_t		bCountryCode;			//< Numeric expression identifying country code of the localized hardware.
	uint8_t		bNumDescriptors;		//< Numeric expression specifying the number of class descriptors (always at least one i.e. Report descriptor.)
	uint8_t		bDescriptorType0;		//< Constant name identifying type of class descriptor. See Section 7.1.2: Set_Descriptor Request for a table of class descriptor constants.
	uint16_t	wDescriptorLength0;		//< Numeric expression that is the total size of the Report descriptor.
} __attribute__((packed));

int16_t init_HID(USBDevice_t *device);

#endif
