/******************************************************************************\
* Projekt:                                                                     *
* Ziel-System:                                                                 *
* Autor: Marco Kaminski                                                        *
* Version: v0.a.001                                                            *
* Datum: 12.01.2013                                                            *
*                                                                              *
* Beschreibung:                                                                *
* Treiber für HID-Devices. (Tatstaur)                                          *
*                                                                              *
* Changelog:                                                                   *
* v0.a.001 - Erste Version                                                     *
*                                                                              *
* Lizenz:                                                                      *
* Dieses Programm ist freie Software. Sie können es unter den Bedingungen      *
* der GNU General Public License, wie von der Free Software Foundation         *
* veröffentlicht, weitergeben und/oder modifizieren, entweder gemäß Version    *
* 2 der Lizenz oder (nach Ihrer Option) jeder späteren Version.                *
*                                                                              *
* Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es        *
* Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die    *
* implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN          *
* BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.      *
*                                                                              *
* Sie sollten eine Kopie der GNU General Public License zusammen mit diesem    *
* Programm erhalten haben.                                                     *
* Falls nicht, schreiben Sie an die Free Software Foundation,                  *
* Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.                *
\******************************************************************************/
#include <stdio.h>

#include "global_inc.h"
#include "usb.h"
#include "HID.h"
#include "ohci.h"

extern				uint8_t	*TDBuffer;
extern volatile	HCCTRL	*HcCtrl;
extern volatile	HCGTD	*TDTail;
extern volatile	HCCA	*Hcca;

        volatile	HCGTD	*TDInt1		= (volatile  HCGTD   *)MEM_TDINT1;
        volatile	HCED	*EDInt1		= (volatile  HCED    *)MEM_EDINT1;
        volatile	uint8_t	*HIDBuffer	= (volatile  uint8_t *)MEM_HIDBUFFER;

int16_t init_HID(USBDevice_t *device) {
	int16_t rc = 0;
	uint8_t b;
	uint16_t descrLength;

	// HID-Descriptor auslesen
	descrLength = 9;
	rc = ControlTransfer(DEVICE_TO_HOST, REQUEST_TYPE_STANDARD, RECIPIENT_INTERFACE, GET_DESCRIPTOR, (HID_DESCRIPTOR << 8), 0, descrLength, TDBuffer);
	if (rc != OK)
		return rc;

	printf("\r\nHID-Descriptor:\r\n");
	for (b = 0; b < descrLength; b++) { printf("%02x ", TDBuffer[b]); }
	printf("\r\n");
	printf("bLength:              0x%02x\r\n", ((struct HIDDescr*)TDBuffer)->bLength);
	printf("bDescriptorType:      0x%02x\r\n", ((struct HIDDescr*)TDBuffer)->bDescriptorType);
	printf("bcdHID:             0x%04x\r\n", ((struct HIDDescr*)TDBuffer)->bcdHID);
	printf("bCountryCode:         0x%02x\r\n", ((struct HIDDescr*)TDBuffer)->bCountryCode);
	printf("bNumDescriptors:      0x%02x\r\n", ((struct HIDDescr*)TDBuffer)->bNumDescriptors);
	printf("bDescriptorType0:     0x%02x\r\n", ((struct HIDDescr*)TDBuffer)->bDescriptorType0);
	printf("wDescriptorLength0: 0x%04x\r\n", ((struct HIDDescr*)TDBuffer)->wDescriptorLength0);

	// Report Descriptor auslesen
	descrLength = ((struct HIDDescr*)TDBuffer)->wDescriptorLength0;
	rc = ControlTransfer(DEVICE_TO_HOST, REQUEST_TYPE_STANDARD, RECIPIENT_INTERFACE, GET_DESCRIPTOR, (REPORT_DESCRIPTOR << 8), 0, descrLength, TDBuffer);
	if (rc != OK)
		return rc;

	printf("\r\nReport-Descriptor:\r\n");
	for (b = 0; b < descrLength; b++) { printf("%02x ", TDBuffer[b]); }
	printf("\r\n");

	// Get Report
	// TODO: Report-Laenge aus dem Report-Descriptor errechnen
	descrLength = 8;
	rc = ControlTransfer(DEVICE_TO_HOST, REQUEST_TYPE_CLASS, RECIPIENT_INTERFACE, GET_REPORT, (REPORT_INPUT << 8), 0, descrLength, &HIDBuffer[8]);
	if (rc != OK)
		return rc;

	printf("\r\nReport:\r\n");
	for (b = 0; b < descrLength; b++) { printf("%02x ", HIDBuffer[8 + b]); }
	printf("\r\n");

	// Set Report, enable NUM-LOCK LED
	// TODO: Report-Laenge aus dem Report-Descriptor errechnen
	descrLength = 1; HIDBuffer[16] = 0x01;
	rc = ControlTransfer(HOST_TO_DEVICE, REQUEST_TYPE_CLASS, RECIPIENT_INTERFACE, SET_REPORT, (REPORT_OUTPUT << 8), 0, descrLength, &HIDBuffer[16]);
	if (rc != OK)
		return rc;

	// Transport-Descriptor fuer Interrupt Endpunkt erstellen
	((SETUP *)&HIDBuffer[0])->rescipient = RECIPIENT_INTERFACE;
	((SETUP *)&HIDBuffer[0])->type       = REQUEST_TYPE_CLASS;
	((SETUP *)&HIDBuffer[0])->direction  = DEVICE_TO_HOST;
	((SETUP *)&HIDBuffer[0])->request    = GET_REPORT;
	((SETUP *)&HIDBuffer[0])->value      = REPORT_INPUT << 8;
	((SETUP *)&HIDBuffer[0])->index      = 0;
	((SETUP *)&HIDBuffer[0])->length     = 8;
	printf("Get Report: %02x %02x %02x %02x %02x %02x %02x %02x\r\n", HIDBuffer[0], HIDBuffer[1], HIDBuffer[2], HIDBuffer[3], HIDBuffer[4], HIDBuffer[5], HIDBuffer[6], HIDBuffer[7]);

	// IN-Token
    initStruct_HCTD(&TDInt1[0]);
    ((HCGTD *)&TDInt1[0])->BuffEnd		= (uint32_t) (&HIDBuffer[0] + (8 - 1));
    ((HCGTD *)&TDInt1[0])->Control.R	= 1;
    ((HCGTD *)&TDInt1[0])->Control.DP	= TD_IN;
    ((HCGTD *)&TDInt1[0])->Control.DI	= 0;
    ((HCGTD *)&TDInt1[0])->Control.T	= TD_TOGGLE_0;
    ((HCGTD *)&TDInt1[0])->Control.CC	= 0xF;
    ((HCGTD *)&TDInt1[0])->CurrBufPtr	= (uint32_t) &HIDBuffer[0];
    ((HCGTD *)&TDInt1[0])->NextTD		= (uint32_t) &TDInt1[1];
    printf("TDInt1[0]: %08x %08x %08x %08x\r\n", ((uint32_t *)&TDInt1[0])[0], ((uint32_t *)&TDInt1[0])[1], ((uint32_t *)&TDInt1[0])[2], ((uint32_t *)&TDInt1[0])[3]);

	// DATA-Token
    initStruct_HCTD(&TDInt1[1]);
    ((HCGTD *)&TDInt1[1])->BuffEnd		= (uint32_t) (&HIDBuffer[8] + (8 - 1));
    ((HCGTD *)&TDInt1[1])->Control.R	= 1;
    ((HCGTD *)&TDInt1[1])->Control.DP	= TD_IN;
    ((HCGTD *)&TDInt1[1])->Control.DI	= 0;
    ((HCGTD *)&TDInt1[1])->Control.T	= TD_TOGGLE_1;
    ((HCGTD *)&TDInt1[1])->Control.CC	= 0xF;
    ((HCGTD *)&TDInt1[1])->CurrBufPtr	= (uint32_t) &HIDBuffer[8];
    ((HCGTD *)&TDInt1[1])->NextTD		= (uint32_t) &TDInt1[2];
    printf("TDInt1[1]: %08x %08x %08x %08x\r\n", ((uint32_t *)&TDInt1[1])[0], ((uint32_t *)&TDInt1[1])[1], ((uint32_t *)&TDInt1[1])[2], ((uint32_t *)&TDInt1[1])[3]);

	// Handshake-Token
    initStruct_HCTD(&TDInt1[2]);
    ((HCGTD *)&TDInt1[2])->BuffEnd		= (uint32_t) NULL;
    ((HCGTD *)&TDInt1[2])->Control.R	= 1;
    ((HCGTD *)&TDInt1[2])->Control.DP	= TD_IN;
    ((HCGTD *)&TDInt1[2])->Control.DI	= 0;
    ((HCGTD *)&TDInt1[2])->Control.T	= TD_TOGGLE_1;
    ((HCGTD *)&TDInt1[2])->Control.CC	= 0xF;
    ((HCGTD *)&TDInt1[2])->CurrBufPtr	= (uint32_t) NULL;
    ((HCGTD *)&TDInt1[2])->NextTD		= (uint32_t) TDTail;
    printf("TDInt1[2]: %08x %08x %08x %08x\r\n", ((uint32_t *)&TDInt1[2])[0], ((uint32_t *)&TDInt1[2])[1], ((uint32_t *)&TDInt1[2])[2], ((uint32_t *)&TDInt1[2])[3]);

    // Endpoint-Descriptor fuer Interrupt Endpunkt erstellen
    initStruct_HCED(EDInt1);
    ((HCED *) EDInt1)->Control.FA	= device->Address;
	((HCED *) EDInt1)->Control.EN	= 1;
	((HCED *) EDInt1)->Control.MPS	= device->ConfigureList[0].InterfaceList[0].EndpointList[0].wMaxPacketSize;
	((HCED *) EDInt1)->Control.S	= 1;
	((HCED *) EDInt1)->Control.K	= 0;
	((HCED *) EDInt1)->TailTD		= (uint32_t) TDTail;
	((HCED *) EDInt1)->HeadTD.C		= 1;
	((HCED *) EDInt1)->HeadTD.H		= 0;
	((HCED *) EDInt1)->HeadTD.res0	= 0;
	((HCED *) EDInt1)->HeadTD.HeadP	= ((uint32_t) &TDInt1[0]) >> 4;
	((HCED *) EDInt1)->NextTD		= 0;
    printf("EDInt1:    %08x %08x %08x %08x\r\n", ((uint32_t *)EDInt1)[0], ((uint32_t *)EDInt1)[1], ((uint32_t *)EDInt1)[2], ((uint32_t *)EDInt1)[3]);

	// TODO: Polling-Frequenz aus dem Interface-Descriptor nehmen.
	Hcca->IntTable[31] = (uint32_t) EDInt1;
	HcCtrl->PLE = 1;

	return rc;
}
